# coding: utf-8

import os

from simple_report.report import SpreadsheetReport
from m3.actions.exceptions import ApplicationLogicException
import settings as gs_settings

from gisdo import constants as const
from gisdo.models import GisdoUnit
from gisdo.utils import DataDictionary


class ReportFormReport(object):
    """
    """

    def __init__(self, result_name):

        templates = os.path.abspath(os.path.join(os.path.dirname(__file__), "templates"))
        self.template_name = os.path.join(templates, "report_template.xlsx")
        self.result_name = os.path.join(gs_settings.DOWNLOADS_DIR, result_name + ".xlsx")

        self.report = SpreadsheetReport(self.template_name)

    def flush_header(self, full_name, email, phone=''):

        header = self.report.get_section('header')
        header.flush({
            'full_name': full_name,
            'email': email,
            'phone': phone
         })

    def collect_row_data(self, report_form_row, date, ocato):

        mo = report_form_row.unit.get_mo()
        try:
            doo_id = report_form_row.unit.gisdo.doo_identity
        except GisdoUnit.DoesNotExist:
            raise ApplicationLogicException(u'Учреждение c ID=%d не найдено в '
                                            u'модуле "Фед. отчетности"' %
                                            report_form_row.unit.pk)
        row_data = {
            'mo': mo.name,
            'dou': report_form_row.unit.name,
            'date': str(date),
            'okato': ocato,
            'octmo': mo.octmo,
            'doo_id': doo_id
        }

        row_data.update(self.collect_application_data(report_form_row))
        row_data.update(self.collect_queues_data(report_form_row))
        row_data.update(self.collect_enrolled_data(report_form_row))
        row_data.update(self.collect_capacities_data(report_form_row))

        return row_data

    def flush_row_data(self, data):

        row = self.report.get_section('row')

        for portion in data:
            row.flush(portion)

    def build(self):

        self.report.build(self.result_name)

    def collect_application_data(self, report_form_row):

        result = DataDictionary()

        # Cписок кортежей вида (индекс, строка, значение)
        # Нас интересует значение
        application_data = report_form_row.get_applications_view_data()
        # индекс кортежа по которому лежит значение
        data_index = 2

        # Общее количество поданных заявлений на постановку в очередь детей
        for age_category in const.AGE_CATEGORIES_FULL_LIST:

            result['app_common_%s' % age_category] = application_data[result.index][data_index]

        # Общее количество заявлений поданных с портала или при личном обращении
        result['app_delivery_portal'] = application_data[result.index][data_index]
        result['app_delivery_person'] = application_data[result.index][data_index]

        # Общее количество поданных заявлений на постановку в очередь в негосударственные
        for age_category in const.AGE_CATEGORIES_FULL_LIST:

            result['app_non_state_%s' % age_category] = application_data[result.index][data_index]

        # Общее количество подданых заявлений на постановку в очередь у ИП
        for age_category in const.AGE_CATEGORIES_FULL_LIST:

            result['app_business_%s' % age_category] = application_data[result.index][data_index]

        # Общее количество поданных заявлений на постановку в очередь в группы
        # кратковременного пребывания
        for age_category in const.AGE_CATEGORIES_CUT_LIST:

            result['app_short_stay_%s' % age_category] = application_data[result.index][data_index]

        result['denied'] = application_data[result.index][data_index]

        return result.get_data()

    def collect_queues_data(self, report_form_row):

        result = DataDictionary()

        # Cписок кортежей вида (индекс, строка, значение)
        # Нас интересует значение
        queues_data = report_form_row.get_queues_view_data()
        # индекс кортежа по которому лежит значение
        data_index = 2

        # Количество детей в очереди (отложенный спрос)
        for age_category in const.AGE_CATEGORIES_FULL_LIST:

            result['queues_common_%s' % age_category] = queues_data[result.index][data_index]

        # Количество детей в очереди (с учетом, желающих сменить ДОО)
        for age_category in const.AGE_CATEGORIES_FULL_LIST:

            result['queues_want_change_doo_%s' % age_category] = queues_data[result.index][data_index]

        # Количество детей в очереди, имеющих право на льготное зачисление
        for age_category in const.AGE_CATEGORIES_FULL_LIST:

            result['queues_benefits_%s' % age_category] = queues_data[result.index][data_index]

        result['queues_federal'] = queues_data[result.index][data_index]
        result['queues_regional'] = queues_data[result.index][data_index]

        # Количество детей в очереди с ограниченными возможностями
        for age_category in const.AGE_CATEGORIES_FULL_LIST:

            result['queues_disable_%s' % age_category] = queues_data[result.index][data_index]

        result['queues_correct'] = queues_data[result.index][data_index]
        result['queues_health'] = queues_data[result.index][data_index]

        # Количество детей в очереди в негосударственные ДОО
        for age_category in const.AGE_CATEGORIES_FULL_LIST:

            result['queues_non_state_%s' % age_category] = queues_data[result.index][data_index]

        # Количество детей в очереди у ИП
        for age_category in const.AGE_CATEGORIES_FULL_LIST:

            result['queues_business_%s' % age_category] = queues_data[result.index][data_index]

        # Количество детей в очереди в группы кратковременного пребывания
        for age_category in const.AGE_CATEGORIES_CUT_LIST:

            result['queues_short_stay_%s' % age_category] = queues_data[result.index][data_index]

        # Количество детей в очереди, необеспеченных местом в ДОО
        for age_category in const.AGE_CATEGORIES_FULL_LIST:

            result['queues_actual_%s' % age_category] = queues_data[result.index][data_index]

        try:
            # Актуальный спрос на текущую дату
            for age_category in const.AGE_CATEGORIES_FULL_LIST:

                result['queues_actual_current_%s' % age_category] = queues_data[result.index][data_index]

            # Отложенный спрос на текущую дату
            for age_category in const.AGE_CATEGORIES_FULL_LIST:

                result['queues_common_current_%s' % age_category] = queues_data[result.index][data_index]
        except IndexError:
            # Отчет старый и в него не входят эти показатели.
            pass

        return result.get_data()

    def collect_enrolled_data(self, report_form_row):

        result = DataDictionary()

        # Cписок кортежей вида (индекс, строка, значение)
        # Нас интересует значение
        enrolled_data = report_form_row.get_enrolled_view_data()
        # индекс кортежа по которому лежит значение
        data_index = 2

        for age_category in const.AGE_CATEGORIES_FULL_LIST:

            result['enrolled_common_%s' % age_category] = enrolled_data[result.index][data_index]

        result['enrolled_want_change_dou'] = enrolled_data[result.index][data_index]
        result['enrolled_family'] = enrolled_data[result.index][data_index]

        for age_category in const.AGE_CATEGORIES_FULL_LIST:

            result['enrolled_disabled_%s' % age_category] = enrolled_data[result.index][data_index]

        result['enrolled_correct'] = enrolled_data[result.index][data_index]
        result['enrolled_health'] = enrolled_data[result.index][data_index]

        for age_category in const.AGE_CATEGORIES_FULL_LIST:

            result['enrolled_non_state_%s' % age_category] = enrolled_data[result.index][data_index]

        for age_category in const.AGE_CATEGORIES_FULL_LIST:

            result['enrolled_business_%s' % age_category] = enrolled_data[result.index][data_index]

        for age_category in const.AGE_CATEGORIES_CUT_LIST:

            result['enrolled_short_stay_%s' % age_category] = enrolled_data[result.index][data_index]

        for age_category in const.AGE_CATEGORIES_CUT_LIST:

            result['enrolled_care_non_state_%s' % age_category] = enrolled_data[result.index][data_index]

        for age_category in const.AGE_CATEGORIES_CUT_LIST:

            result['enrolled_care_business_%s' % age_category] = enrolled_data[result.index][data_index]

        for age_category in const.AGE_CATEGORIES_FULL_LIST:

            result['enrolled_inline_enrolled_%s' % age_category] = enrolled_data[result.index][data_index]

        # В отчет не попадает показатель "Общее количество детей ..." в 15 возрастных категориях
        result.index = result.index + 15
        try:
            for age_category in const.AGE_CATEGORIES_FULL_LIST:

                result['enrolled_common_current_%s' % age_category] = enrolled_data[result.index][data_index]
        except IndexError:
            # Отчет старый и в него не входят эти показатели.
            pass

        return result.get_data()

    def collect_capacities_data(self, report_form_row):

        result = DataDictionary()

        capacities_data = report_form_row.get_capacities_view_data()
        data_index = 2

        for age_category in const.AGE_CATEGORIES_CUT_LIST:

            result['capacities_common_%s' % age_category] = capacities_data[result.index][data_index]

        for age_category in const.AGE_CATEGORIES_CUT_LIST:

            result['capacities_disabled_%s' % age_category] = capacities_data[result.index][data_index]

        for age_category in const.AGE_CATEGORIES_CUT_LIST:

            result['capacities_non_state_%s' % age_category] = capacities_data[result.index][data_index]

        for age_category in const.AGE_CATEGORIES_CUT_LIST:

            result['capacities_business_%s' % age_category] = capacities_data[result.index][data_index]

        for age_category in const.AGE_CATEGORIES_CUT_LIST:

            result['capacities_short_stay_%s' % age_category] = capacities_data[result.index][data_index]

        return result.get_data()